# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os, gtk, vte, pango
import child, __BASIC__, GConf, Conf, func, i18n, Menu

_ = i18n.gettext()

class egg (__BASIC__.egg) :

	WIDGET = None

	CHILD_TYPE = child.TYPE_VIM

	ARGUMENT = tuple()
	DIRECTORY = str()
	FORK_COMMAND = str()
	ENVIRONMENT = tuple()

	DEFAULT_FONTNAME = "monospace 16"
	EMULATION = "xterm"
	UNIT_FONT_DELM = 1000
	RANGE_FONT_SIZE = (10000, 30000)
	SIGNAL_REMOVE = "child-exited"
	MAX_SCROLL_LINE = 100
	EXTERNAL_COMMAND = str()
	ID_NOTIFY = None
	SETTING = dict()

	PID = None

	def __init__ (self, **kwargs) :

		__BASIC__.egg.__init__(self, **kwargs)

		if kwargs.has_key("environment") :
			if type(kwargs["environment"]) is not tuple :
				raise TypeError, _("The type of environment must be tuple.")

			self.ENVIRONMENT = kwargs["environment"]

		if kwargs.has_key("argument") :
			if type(kwargs["argument"]) is not tuple :
				raise TypeError, _("The type of argument must be tuple.")

			self.ARGUMENT = kwargs["argument"]

		if kwargs.has_key("directory") :
			if os.path.isdir(kwargs["directory"]) :
				__dirname = kwargs["directory"]
			else :
				__dirname = os.path.dirname(kwargs["directory"])

			self.DIRECTORY = __dirname

	def make (self) :

		self.WIDGET = vte.Terminal()

		# DEFAULT SETTING.
		self.WIDGET.set_audible_bell(False)
		self.WIDGET.set_visible_bell(True)
		self.WIDGET.set_scroll_on_output(False)

		if self.SETTING.has_key("max-scroll-line") and \
				type(self.SETTING["max-scroll-line"]) is int :
			__max_scroll_line = self.SETTING["max-scroll-line"]
		else :
			__max_scroll_line = \
				self.GCONF_CLIENT.get_value_by_conf("max-scrollline")

		self.WIDGET.set_scrollback_lines(__max_scroll_line)
		self.WIDGET.set_mouse_autohide(True)
		#self.WIDGET.set_size(100,300)
		self.set_color()
		self.set_notify()

		# set font
		__fontname = self.get_font()
		self.set_font_from_string(__fontname)

		self.WIDGET.connect("char-size-changed", self.reset)
		self.reset()

		self.fork_command()

		self.WIDGET.show()
		self.WIDGET.grab_remove()

		self.activate_dnd()
		self.attach_popupmenu()

	def fork_command (self) :
		__argument = (self.FORK_COMMAND, ) + self.ARGUMENT
		__environment = ("TERM=%s" % self.EMULATION, ) + self.ENVIRONMENT

		self.PID = self.WIDGET.fork_command( \
			self.FORK_COMMAND, \
			argv=__argument, \
			envv=__environment, \
			directory=self.DIRECTORY, \
		)

	def get_font (self) :
		if self.SETTING.has_key("use-system-default-font") and \
				self.SETTING["use-system-default-font"] == True :
			__fontname = \
				self.GCONF_CLIENT.get_value(Conf.GLOBAL_FONT_SYSTEM_DEFAULT)
		elif self.SETTING.has_key("font") and self.SETTING["font"] :
			__fontname = self.SETTING["font"]
		else :
			__fontname = \
				self.GCONF_CLIENT.get_value_by_conf("font")

		if not __fontname.strip() :
			__fontname = self.DEFAULT_FONTNAME

		return __fontname

	def reset (self, parm1=None, parm2=None, parm3=None) :
		self.WIDGET.reset(True, False)

	def set_font_from_string (self, fontname) :
		self.WIDGET.set_font_from_string(fontname)
		self.reset()

	def set_font (self, fontDescription) :
		self.WIDGET.set_font(fontDescription)
		self.reset()

	def decrease_font (self, size=int()) :

		pd = self.WIDGET.get_font()

		__s = pd.get_size() - self.UNIT_FONT_DELM
		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)
		self.reset()

		return

	def increase_font (self, size=int()) :

		pd = self.WIDGET.get_font()

		__s = pd.get_size() + self.UNIT_FONT_DELM
		if __s not in \
				range(self.RANGE_FONT_SIZE[0], self.RANGE_FONT_SIZE[1]) :
			return

		pd.set_size(__s)
		self.WIDGET.set_font(pd)
		self.reset()

		return

	def attach_popupmenu (self) :
		callback_functions = {"reset" : self.reset}
		self.WIDGET.connect( \
			"button-press-event", \
			Menu.on_show_popup_vte, \
			self.WIDGET, \
			callback_functions \
		)

	def on_change_color (self, client, parm1, entry, type) :

		__c = self.GCONF_CLIENT.get_value_from_value(entry.get_value())
		__c0 = func.str_to_color(__c)

		try :
			__func = getattr(self.WIDGET, "set_color_%s" % type)
		except AttributeError :
			raise AttributeError, "What type, % is it?" % type
		else :
			__func(__c0)

	def set_color (self) :

		# initiate terminal color set.
		self.WIDGET.set_default_colors()

		if self.SETTING.has_key("color-foreground") :
			__c = self.SETTING["color-foreground"]
		else :
			__c = self.GCONF_CLIENT.get_value( \
				Conf.SCHEMA["color-foreground"]["key"])

		__c0 = func.str_to_color(__c)
		self.WIDGET.set_color_foreground(__c0)

		if self.SETTING.has_key("color-background") :
			__c = self.SETTING["color-background"]
		else :
			__c = self.GCONF_CLIENT.get_value( \
				Conf.SCHEMA["color-background"]["key"])
		__c0 = func.str_to_color(__c)
		self.WIDGET.set_color_background(__c0)

		if self.SETTING.has_key("color-dim") :
			__c = self.SETTING["color-dim"]
		else :
			__c = self.GCONF_CLIENT.get_value( \
				Conf.SCHEMA["color-dim"]["key"])
		__c0 = func.str_to_color(__c)
		self.WIDGET.set_color_dim(__c0)

		if self.SETTING.has_key("color-highlight") :
			__c = self.SETTING["color-highlight"]
		else :
			__c = self.GCONF_CLIENT.get_value( \
				Conf.SCHEMA["color-highlight"]["key"])
		__c0 = func.str_to_color(__c)
		self.WIDGET.set_color_highlight(__c0)

		if self.SETTING.has_key("color-cursor") :
			__c = self.SETTING["color-cursor"]
		else :
			__c = self.GCONF_CLIENT.get_value( \
				Conf.SCHEMA["color-cursor"]["key"])
		__c0 = func.str_to_color(__c)
		self.WIDGET.set_color_cursor(__c0)


	def send (self, command) :
		self.WIDGET.feed_child(command)

	def on_drag_received \
			(self, widget, context, x, y, selection, targetType, time) :

		__path = func.get_filename_from_uri(selection.data)
		self.send(__path)

		return

		if targetType == Conf.DND_TARGET_COLOR :
			pass
		elif targetType == Conf.DND_TARGET_BGIMAGE :
			pass
		elif targetType == Conf.DND_TARGET_RESET_BG :
			pass
		elif targetType == Conf.DND_TARGET_URI_LIST :
			pass
		elif targetType == Conf.DND_TARGET_MOZ_URL :
			pass
		elif targetType == Conf.DND_TARGET_UTF8_STRING :
			pass
		elif targetType == Conf.DND_TARGET_COMPOUND_TEXT :
			pass
		elif targetType == Conf.DND_TARGET_TEXT_PLAIN :
			pass
		elif targetType == Conf.DND_TARGET_STRING :
			pass
		elif targetType == Conf.DND_TARGET_TEXT_PLAIN :
			pass
		elif targetType == Conf.DND_TARGET_TEXT :
			pass
		else :
			pass

	def exit (self) :
		if self.ID_NOTIFY :
			self.GCONF_CLIENT.CLIENT.notify_remove(self.ID_NOTIFY)

class tab (__BASIC__.tab) :

	def __init__ (self, state, **kwargs) :
		__BASIC__.tab.__init__(self, state, **kwargs)

		if self.KWARGS.has_key("environment") :
			self.KWARGS["environment"] = \
				("UID_VEE=%s" % self.STATE.DATA["uid"]) + \
				self.KWARGS["environment"]
		else :
			self.KWARGS["environment"] = \
				("UID_VEE=%s" % self.STATE.DATA["uid"], )

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "__VTE__.py"


